unit uoz_sys;
interface

uses
  Classes, SysUtils, Controls, ToolWin, ComCtrls, jwawinbase, jwawinnt, SdpoSerial;

Const
   AutoSaveTime=480;     // Aвтосохранение через 2 минуты  {60 секунд = 250 мс*240}
   PCount         =$20;
   UResetUART     = $0;  // Сброс UART
   UGetParam      = $1;  // Выдать параметры: NCC(HL) Selectr UOZSelectr Status EStatus UOZCurrent(HL) TableAdress
   UGetActiveTable= $6;  // Выдать активную таблицу УОЗ
   UGetIndex      = $8;  // Выдать таблицу Index
   UGetTester     = $9;  // Выдать массив тестовых данных (8 байт) для отладки
   USaveAllTable  = $A;  // Сохранить все таблицы  в FLASH
   UClearUARTCtrl = $B;  // Автоматическое управление активной страницей (прошивка)
   ULoadAllTable  = $C;  // Загрузить все таблицы  из FLASH c проверкой на допустимые значения
   UClearAllTable = $E;  // Заполнить (начальное значение) все таблицы (без записи в FLASH)
   USetUARTCtrl   = $D;  // Управление активной страницей по UART
   UGetConst      = $F;  // Выдать константы: Версия прошивки, OUZMinFP8, OUZMaxFP8, LDelta, Base(HL), Fn, Ka, UOZCount, LDeltaFP16(HL)
   USetUOZL       = $20; // Записать младший полубайт в активную таблицу УОЗ по текущему адресу
   USetUOZH       = $30; // Записать старший полубайт в активную таблицу УОЗ по текущему адресу
   USetAddr       = $40; // +N Установить текущий адрес записи в активной таблице.
   UGetTable      = $60; // +N Выдать таблицу N
   USetActiveTable= $80; // +N Установить активной таблицу N
   USaveTable     = $A0; // +N Сохранить таблицу N в FLASH
   ULoadTable     = $C0; // +N Загрузить таблицу N из FLASH c проверкой на допустимые значения
   UClearTable    = $E0; // +N Заполнить (начальное значение) таблицу с номером N(без записи в FLASH)
   Buffsize       = 4*1024;
Type
  TSvect   = array [0..255] of string;
  TBvect   = array [0..PCount-1] of byte;
  TWvect   = array [0..PCount-1] of word;
  TIvect   = array [0..PCount-1] of integer;
  TEvect   = array [0..PCount-1] of extended;

  TDT=record
     Cnt:longint;
     Delay:boolean;
     end;

  //  OUZ = Ka * [LDelta-Alfa] / [Base-LDelta]          [1]

 TCore=record
     Version:byte;
     LDelta:extended;      // Угловой размер метки LDelta в градусах
     LDeltaFP16:word;     // Дробная часть угловой размера метки LDelta в градусах
     Base:word;           // Угол-база в градусах
     Fn:byte;             // Код частоты тактовых импульсов
     Ka:byte;             // Масштабирующий коэффициент
     OUZMaxFP8:byte;      // Число в формате FP8 [1] при Alfa=0
     OUZMinFP8:byte;      // Число в формате FP8 [1] при Alfa=AlfaMax < LDelta
     UOZCount:byte;       // Количество таблиц
     LoadConst:boolean;   // Загрузка параметров произведена

     Index:  TWvect;      // Таблица границ диапазонов
     LoadIndex:boolean;   // Загрузка таблицы Index произведена

     NCC:word;            //значение счетчика-измерителя числа оборотов
     Selectr:byte;        //Селектр в таблице оборотов OUZ
     UOZSelectr:byte;     //Номер активной таблицы OUZ
     Status:byte;         //Регистр флагов состояния
                          // 7	LowMode - режим низкие обороты
                          // 6	StopMode - режим остановки
                          // 5	UOZMax - расчетый OUZ ограничен сверху (Угол ограничен снизу)
                          // 4	OUZMin - расчетый OUZ ограничен снизу (Угол ограниxен сверху)
                          // 0	UARTCtrl - Флаг управления UOZSelectr: 0 - прошивка  (авто), 1 - по UART
     LowMode:boolean;     //признак низких оборотов (Status)
     StopMode:boolean;    //признак режима Стоп (Status)
     OUZMax:boolean;      //ограничение в формуле расчета OUZ (Миниммальный угол)
     OUZMin:boolean;      //ограничение в формуле расчета OUZ (Максимальный угол)
     UARTCtrl:boolean;    //Флаг управления UOZSelectr: 0 - прошивка (авто), 1 - по UART

     EStatus:byte;        // Расширенный регистр флагов состояния
                          // RnewCMD		=	0x40
     AllCmdComplite:boolean;//Все команды обработаны блоком
     UOZCurrentFP16:word; //Результирующее значение OUZ
     TableAdress:byte;    //Адрес для чтения/записи Таблиц UOZ по UART
     LoadParam:boolean;   //Загрузка параметров произведена

     UOZ:array [0..PCount-1] of TBvect;     //Таблица UOZ
     UOZLoad:array [0..PCount-1] of boolean;//Таблица UOZ


 //   Производные величины
     AF:extended;         // Коэффициент перевода  кода таймера в число оборотов
                          // AF = 8000000 / 6 * (Base-LDelta) / (1+Fn)
                          // N (1/мин) = AF / NCС
     AlfaMax:extended;    // Максимальный УОЗ
     GFP8:extended;       // (Base-LDelta)/(128*Ka);
     GFP16:extended;      // (Base-LDelta)/(32768*Ka);
     Alfa,LDELTALength:extended;
     No:integer;          //Обороты 1/мин

     UOF:array [0..PCount-1] of TEvect;     //Таблица УОЗ
     tester:array [0..7] of byte;
        end;



var
   Versions:TSvect;
   Core:TCore;
   Tbuff,RBuff:array [0..Buffsize-1] of byte;
   Tbuffptr:word;
   RbuffptrI,RbuffptrO:word;
   NC,NScr: TIVect;
   AScr: array[0..PCount] of TIVect;
   SystemCnt:longint;
   WDT:TDT;
   AutoSave:TDT;
   kk:integer;
   GetStarted, ShowTester:boolean;
   Nmax, Nmin, ymin, ymax, xmax, xmin:integer;
   ax, bx, ay, by:extended;
   ChangeValue:boolean;
   NCCOld:word;
   UOZSelectrOld:byte ;
   UOZCurrentFP16Old:word;
   UOZOld:array [0..PCount-1] of TBvect;


Procedure SendCommand(A:byte);                             //Добавиь команду в буфер передачи
procedure TrasmiteCommand(var Serial: TSdpoSerial);        //ОТправить все команды из буфера передачи
Function  FindCOM(var AA:TSvect):boolean;
Function  incPtr(t:word):word;
Function  addPtr(t,s:word):word;
Function  subPtr(t,s:word):word;
Function  LoadAllCore:boolean;
Function  ChangeCore:boolean;
Procedure SaveOUZ(filename:string);
Function  OUZFP8toAlfa(ab:byte):extended;
Function  OUZFP16toAlfa(ab:word):extended;
Function  AlfatoOUZFP8(ab:extended):byte;

implementation



Function OUZFP16toAlfa(ab:word):extended;
var  aa:extended;
begin
with Core do aa:=LDelta-ab*GFP16;
if aa<0 then aa:=0;
OUZFP16toAlfa:=aa;
end;


Function OUZFP8toAlfa(ab:byte):extended;
var  aa:extended;
begin
with Core do aa:=LDelta-ab*GFP8;
if aa<0 then aa:=0;
OUZFP8toAlfa:=aa;
end;


Function AlfatoOUZFP8(ab:extended):byte;
var ac:byte;
begin
with Core do
begin
ac:=round((LDelta-ab)/GFP8);
if ac>Core.OUZMaxFP8 then ac:=Core.OUZMaxFP8;
if ac<Core.OUZMinFP8 then ac:=Core.OUZMinFP8;
end;
AlfatoOUZFP8:=ac;
end;


Procedure SaveOUZ(filename:string);
var ff:file of tbvect;
i:integer;
begin
 assignfile(ff,filename);
 rewrite(ff);
 for i:=0 to Core.UOZCount-1 do  write(ff,Core.UOZ[i]);
 closefile(ff);
end;

procedure SendCommand(A:byte);
begin
  Tbuff[Tbuffptr]:=A;
  inc(Tbuffptr);
end;

procedure TrasmiteCommand(var Serial: TSdpoSerial);
begin
  if Tbuffptr=0 then exit;
  Serial.WriteBuffer(Tbuff,Tbuffptr); Tbuffptr:=0;
  Core.AllCmdComplite:=false;
end;

Function FindCOM(var AA:Tsvect):boolean;
var
   i:Integer;
   Phandle:Thandle;
   ii:boolean;
begin
ii:=false;
for i:=1 to 10 do
    begin
      Phandle:=CreateFile(Pchar('COM'+intToStr(i)), Generic_Read or Generic_Write, 0, nil,open_existing, file_flag_overlapped,0);
      if Phandle<>invalid_handle_value then
                                         begin
                                          ii:=true;
                                          AA[i]:='COM'+intToStr(i);
                                          CloseHandle(Phandle);
                                         end
                                       else AA[i]:='';
    end;
FindCOM:=ii;
end;

Function  incPtr(t:word):word;
begin
    incPtr:=(t+1) and (Buffsize-1);
end;

Function  addPtr(t,s:word):word;
begin
    addPtr:=(t+s) and (Buffsize-1);
end;

Function  subPtr(t,s:word):word;
begin
     if t<s then subPtr:=Buffsize-(s-t)
            else subPtr:=t-s;
end;


Function  LoadAllCore:boolean;
var i:integer;
    aa:boolean;
begin
  if not Core.LoadConst then
                          begin
                            LoadAllCore:=false;
                            exit;
                          end;
  if not Core.LoadIndex then
                          begin
                            LoadAllCore:=false;
                            exit;
                          end;
  aa:=Core.UOZLoad[0];
  for i:=1 to Core.UOZCount-1 do aa:=aa and Core.UOZLoad[i];
  LoadAllCore:=aa;
end;

Function ChangeCore:boolean;
var j,i:integer;
    aa:boolean;
begin
if Core.UOZSelectr<>UOZSelectrOld then
                      begin
                        ChangeCore:=true;
                        exit;
                      end;
if Core.NCC<>NCCOld then
                      begin
                        ChangeCore:=true;
                        exit;
                      end;
if Core.UOZCurrentFP16<>UOZCurrentFP16Old then
                      begin
                        ChangeCore:=true;
                        exit;
                      end;
j:=0;
repeat
aa:=false;
for i:=0 to PCount-1 do
    if Core.UOZ[j,i]<>UOZOld[j,i] then aa:=true;
if aa then
                       begin
                         ChangeCore:=true;
                         exit;
                       end;
inc(j)
until j=Core.UOZCount;
ChangeCore:=false;
end;



Begin
ShowTester:=false;
if paramcount>0 then
                if lowercase(paramstr(1))='tester' then ShowTester:=true;

Versions[0]:='........';
for kk:=1 to 255 do Versions[kk]:='Нет информации';
with Core do
     begin
       Version:=0;
       LoadConst:=false; LoadParam:=false;
       LoadIndex:=false;
       for kk:=0 to PCount-1 do UOZLoad[kk]:=false;
     end;
ChangeValue:=false;
Tbuffptr:=0;
RbuffptrI:=Buffsize-1;
RbuffptrO:=Buffsize-1;
WDT.Delay:=false;
AutoSave.Cnt:=AutoSaveTime;
SystemCnt:=0;
AutoSave.Delay:=true;  //Автосохранение на диск включено
Versions[10]:='Ветерок МЛ-10 04.10.2020';

Versions[20]:='Устарела!!! октябрь 2020';
Versions[21]:='МС-40 28.11.2020'; // версия => с корректором и TCI выходом

Versions[30]:='Устарела!!! октябрь 2020';
Versions[31]:='Устарела!!! октябрь 2020';
Versions[32]:='Устарела!!! ноябрь 2020'; // версия => с корректором
Versions[33]:='Одноцилиндровый двигатель 28.11.2020'; // версия => с корректором и TCI выходом

Versions[40]:='Трехцилиндровый снегоход';

Versions[50]:='Трехцилиндровый Вихрь';
end.





